@@ -1,7 +1,5 @@
 #include "qmlutils.h"
-#include <QApplication>
 #include <QDateTime>
-#include <QFile>
 
 QmlUtils::QmlUtils(QObject *parent) :
     QObject(parent)
@@ -39,16 +37,3 @@ qint64 QmlUtils::currentDateTime() {
 QString QmlUtils::localDateTimeFromString(const QString &dateTimeString, Qt::DateFormat format) {
     return Utils::localDateTimeFromString(dateTimeString, format);
 }
-
-QString QmlUtils::readPrivacyPolicy() {
-    QString fileName = QApplication::applicationDirPath().section('/', 0, -2) + "/resources/qdl_privacy_policy.html";
-    QFile aFile(fileName);
-    QString policy;
-
-    if (aFile.open(QIODevice::ReadOnly)) {
-        policy = aFile.readAll();
-        aFile.close();
-    }
-
-    return policy;
-}
