@@ -8,9 +8,6 @@ class QmlUtils : public QObject
 {
     Q_OBJECT
 
-    Q_PROPERTY(QString versionNumber
-               READ versionNumber
-               CONSTANT)
 public:
     explicit QmlUtils(QObject *parent = 0);
     Q_INVOKABLE QString durationFromSecs(int secs);
@@ -21,8 +18,6 @@ public:
     Q_INVOKABLE QString dateFromMSecs(qint64 msecs, bool showTime = true);
     Q_INVOKABLE qint64 currentDateTime();
     Q_INVOKABLE QString localDateTimeFromString(const QString &dateTimeString, Qt::DateFormat = Qt::ISODate);
-    inline QString versionNumber() const { return VERSION_NUMBER; }
-    Q_INVOKABLE QString readPrivacyPolicy();
 };
 
 #endif // QMLUTILS_H
