@@ -31,10 +31,13 @@
 #include <QWidgetAction>
 #include <QProgressDialog>
 #include <QTimer>
+#include <QMaemo5InformationBox>
+#include <QPushButton>
 
 MainWindow::MainWindow(QWidget *parent) :
     QMainWindow(parent),
     m_filterAction(new ValueSelectorAction(this)),
+    m_nextAction(new ValueSelectorAction(this)),
     m_concurrentTransfersAction(new ValueSelectorAction(this)),
     m_connectionsAction(new ValueSelectorAction(this)),
     m_rateLimitAction(new ValueSelectorAction(this)),
@@ -61,13 +64,18 @@ MainWindow::MainWindow(QWidget *parent) :
     m_transferPriorityAction(m_contextMenu->addAction(tr("Priority"), this, SLOT(showTransferPriorityDialog()))),
     m_transferRemoveAction(m_contextMenu->addAction(tr("Remove"), this, SLOT(removeCurrentTransfer()))),
     m_checkDialog(new CheckUrlsDialog(this)),
-    m_progressDialog(new QProgressDialog(this))
+    m_progressDialog(new QProgressDialog(this)),
+    m_cancelButton(new QPushButton(tr("Cancel"), m_progressDialog))
 {
     this->setWindowTitle(tr("QDL"));
     this->setCentralWidget(m_view);
     this->setAttribute(Qt::WA_Maemo5StackedWindow, true);
     this->addToolBar(Qt::BottomToolBarArea, m_toolBar);
 
+    m_nextAction->setText(tr("After current download(s)"));
+    m_nextAction->setModel(new TransferActionModel(this));
+    m_nextAction->setValue(Transfers::Continue);
+
     m_filterAction->setText(tr("Show"));
     m_filterAction->setModel(new StatusFilterModel(this));
     m_filterAction->setValue(Transfers::Unknown);
@@ -85,6 +93,7 @@ MainWindow::MainWindow(QWidget *parent) :
     m_rateLimitAction->setValue(Settings::instance()->downloadRateLimit());
 
     this->menuBar()->addAction(m_filterAction);
+    this->menuBar()->addAction(m_nextAction);
     this->menuBar()->addAction(m_concurrentTransfersAction);
     this->menuBar()->addAction(m_connectionsAction);
     this->menuBar()->addAction(m_rateLimitAction);
@@ -123,6 +132,9 @@ MainWindow::MainWindow(QWidget *parent) :
 
     m_transferConvertToAudioAction->setCheckable(true);
 
+    m_progressDialog->setWindowTitle(tr("Please wait"));
+    m_progressDialog->setCancelButton(m_cancelButton);
+
     this->connect(m_filterAction, SIGNAL(valueChanged(QVariant)), this, SLOT(setTransferFilter(QVariant)));
     this->connect(m_concurrentTransfersAction, SIGNAL(valueChanged(QVariant)), this, SLOT(setMaximumConcurrentTransfers(QVariant)));
     this->connect(m_connectionsAction, SIGNAL(valueChanged(QVariant)), this, SLOT(setGlobalTransferConnections(QVariant)));
@@ -150,6 +162,7 @@ MainWindow::MainWindow(QWidget *parent) :
     this->connect(UrlRetriever::instance(), SIGNAL(progressChanged(int)), this, SLOT(updateProgressDialog(int)));
     this->connect(UrlRetriever::instance(), SIGNAL(finished()), this, SLOT(onUrlRetrieverFinished()));
     this->connect(UrlRetriever::instance(), SIGNAL(finished()), this, SLOT(hideProgressDialog()));
+    this->connect(UrlRetriever::instance(), SIGNAL(cancelled()), this, SLOT(hideProgressDialog()));
 
     this->onPackageCountChanged(m_model->rowCount());
 }
@@ -169,6 +182,10 @@ void MainWindow::onPluginsReady() {
     m_model->restoreStoredTransfers();
 }
 
+void MainWindow::setNextAction(const QVariant &value) {
+    m_model->setNextAction(static_cast<Transfers::Action>(value.toInt()));
+}
+
 void MainWindow::setTransferFilter(const QVariant &value) {
     m_model->setStatusFilter(static_cast<Transfers::Status>(value.toInt()));
 }
@@ -191,6 +208,7 @@ void MainWindow::updateSpeed(int speed) {
 
 void MainWindow::onPackageCountChanged(int count) {
     m_filterAction->setEnabled(count > 0);
+    m_nextAction->setEnabled(count > 0);
     m_transferPropertiesAction->setEnabled(count > 0);
     m_packagePropertiesAction->setEnabled(count > 0);
     m_startAction->setEnabled(count > 0);
@@ -344,17 +362,15 @@ void MainWindow::showAddUrlsDialog(const QString &text, const QString &fileName)
 }
 
 void MainWindow::onUrlRetrieverFinished() {
-    QString text;
+    QString results = UrlRetriever::instance()->resultsString();
 
-    foreach (QUrl url, UrlRetriever::instance()->results()) {
-        text.append(url.toString() + "\n");
+    if (results.isEmpty()) {
+        QMaemo5InformationBox::information(this, tr("No supported URLs found"));
     }
-
-    if (!text.isEmpty()) {
-        this->showAddUrlsDialog(text);
+    else {
+        this->showAddUrlsDialog(results);
+        UrlRetriever::instance()->clearResults();
     }
-
-    UrlRetriever::instance()->clearResults();
 }
 
 void MainWindow::showRetrieveUrlsDialog(const QString &text, const QString &fileName) {
@@ -393,10 +409,10 @@ void MainWindow::showProgressDialog(const QString &message, int maximum) {
     m_progressDialog->setRange(0, maximum);
 
     if ((this->sender()) && (this->connect(m_progressDialog, SIGNAL(canceled()), this->sender(), SLOT(cancel())))) {
-        m_progressDialog->setCancelButtonText(tr("Cancel"));
+        m_cancelButton->setEnabled(true);
     }
     else {
-        m_progressDialog->setCancelButtonText(QString());
+        m_cancelButton->setEnabled(false);
     }
 
     m_progressDialog->open();
