@@ -5,9 +5,9 @@
 #include <QList>
 #include <QSize>
 
-static const QString VERSION_NUMBER("1.0.2");
+static const QString VERSION_NUMBER("1.0.3");
 static const QString ICON_PATH("/opt/qdl/icons/");
-#ifdef Q_WS_MAEMO_5
+#if (defined Q_WS_MAEMO_5) || (defined MAEMO4_OS)
 static const QSize ICON_SIZE(36, 36);
 #else
 static const QSize ICON_SIZE(16, 16);
