@@ -10,7 +10,6 @@ class ScreenOrientation : public QObject
 
     Q_ENUMS(Orientation)
 
-public:
 public:
     enum Orientation {
 #ifdef SAILFISH_OS
@@ -50,7 +49,7 @@ class Transfers : public QObject
 {
     Q_OBJECT
 
-    Q_ENUMS(Status Priority)
+    Q_ENUMS(Status Priority Action)
 
 public:
     enum Status {
@@ -74,6 +73,12 @@ public:
         LowPriority
     };
 
+    enum Action {
+        Continue = 0,
+        Pause,
+        Quit
+    };
+
     inline static QString statusString(Status status) {
         switch (status) {
         case Queued:
@@ -115,6 +120,19 @@ public:
             return QString();
         }
     }
+
+    inline static QString actionString(Action action) {
+        switch (action) {
+        case Continue:
+            return tr("Continue");
+        case Pause:
+            return tr("Pause");
+        case Quit:
+            return tr("Quit");
+        default:
+            return QString();
+        }
+    }
 };
 
 class NetworkProxyType : public QObject
@@ -129,6 +147,19 @@ public:
         HttpProxy = QNetworkProxy::HttpProxy,
         HttpCachingProxy = QNetworkProxy::HttpCachingProxy
     };
+
+    inline static QString networkProxyTypeString(ProxyType type) {
+        switch (type) {
+        case Socks5Proxy:
+            return QString("SOCKS5");
+        case HttpProxy:
+            return QString("HTTP");
+        case HttpCachingProxy:
+            return QString("HTTP %1").arg(tr("caching"));
+        default:
+            return QString("HTTP");
+        }
+    }
 };
 
 #endif // ENUMS_H
