@@ -44,6 +44,11 @@ void SelectionModel::addItem(const QString &name, const QVariant &value) {
     emit countChanged(this->rowCount());
 }
 
+void SelectionModel::clear() {
+    QStandardItemModel::clear();
+    emit countChanged(this->rowCount());
+}
+
 ScreenOrientationModel::ScreenOrientationModel(QObject *parent) :
     SelectionModel(parent)
 {
@@ -294,6 +299,14 @@ StatusFilterModel::StatusFilterModel(QObject *parent) :
     this->addItem(tr("Failed"), Transfers::Failed);
 }
 
+TransferActionModel::TransferActionModel(QObject *parent) :
+    SelectionModel(parent)
+{
+    this->addItem(tr("Continue"), Transfers::Continue);
+    this->addItem(tr("Pause"), Transfers::Pause);
+    this->addItem(tr("Quit"), Transfers::Quit);
+}
+
 NetworkProxyTypeModel::NetworkProxyTypeModel(QObject *parent) :
     SelectionModel(parent)
 {
