@@ -8,7 +8,6 @@
 #include "pluginmanager.h"
 #include "clipboardmonitor.h"
 #include "transfermodel.h"
-#include <QThread>
 
 Session::Session(QObject *parent) :
     QObject(parent),
@@ -20,26 +19,17 @@ Session::Session(QObject *parent) :
     m_urlRetriever(UrlRetriever::instance()),
     m_pluginManager(PluginManager::instance()),
     m_clipboardMonitor(ClipboardMonitor::instance()),
-    m_model(TransferModel::instance()),
-    m_thread(new QThread)
+    m_model(TransferModel::instance())
 {
     m_nam->setParent(this);
     m_settings->setParent(this);
+    m_database->setParent(this);
+    m_storage->setParent(this);
     m_urlChecker->setParent(this);
     m_urlRetriever->setParent(this);
     m_pluginManager->setParent(this);
     m_clipboardMonitor->setParent(this);
     m_model->setParent(this);
-
-    m_database->moveToThread(m_thread);
-    m_storage->moveToThread(m_thread);
-    m_thread->start(QThread::LowPriority);
-
-    this->connect(m_thread, SIGNAL(finished()), m_thread, SLOT(deleteLater()));
-    this->connect(m_thread, SIGNAL(finished()), m_database, SLOT(deleteLater()));
-    this->connect(m_thread, SIGNAL(finished()), m_storage, SLOT(deleteLater()));
 }
 
-Session::~Session() {
-    m_thread->quit();
-}
+Session::~Session() {}
