@@ -14,7 +14,6 @@ class PluginManager;
 class NetworkAccessManager;
 class ClipboardMonitor;
 class TransferModel;
-class QThread;
 
 class Session : public QObject
 {
@@ -34,7 +33,6 @@ private:
     PluginManager *m_pluginManager;
     ClipboardMonitor *m_clipboardMonitor;
     TransferModel *m_model;
-    QThread *m_thread;
 };
 
 #endif // SESSION_H
