@@ -9,8 +9,6 @@ class Storage : public QObject
     Q_OBJECT
 
 public:
-    ~Storage();
-
     static Storage* instance();
 
     QString directory() const;
@@ -19,12 +17,13 @@ public:
     QString errorString() const;
 
 public slots:
-    bool storeTransfers(QList<Transfer*> transfers);
+    bool storeTransfers(QList<Transfer*> transfers, bool deleteWhenStored);
     bool restoreTransfers();
     bool clearStoredTransfers();
     
 private:
-    explicit Storage();
+    Storage();
+    ~Storage();
 
     void setErrorString(const QString &errorString);
 
