@@ -15,6 +15,15 @@
 #include <QDateTime>
 #include <QNetworkReply>
 #include <QDir>
+#include <qplatformdefs.h>
+#ifdef MEEGO_EDITION_HARMATTAN
+#include "../harmattan/captchaimageprovider.h"
+#include "../harmattan/qmlutils.h"
+#include <QDeclarativeView>
+#include <QDeclarativeEngine>
+#include <QDeclarativeContext>
+#include <QGraphicsObject>
+#endif
 
 static const qint64 MIN_FRAGMENT_SIZE = 1024 * 2048;
 static const QString VIDEO_SUFFIXES("mp4:flv:avi:divx:mpg:mpeg:mpeg2:mpeg4:ts:mkv:wmv:xvid:mov");
@@ -43,6 +52,9 @@ Transfer::Transfer(QObject *parent) :
     m_row(0),
     m_preferredConnections(1),
     m_maxConnections(1)
+  #ifdef QML_USER_INTERFACE
+    ,m_expanded(false)
+  #endif
 {
     this->connect(&m_file, SIGNAL(writeCompleted()), this, SLOT(onFileWriteCompleted()));
     this->connect(&m_file, SIGNAL(error()), this, SLOT(onFileError()));
@@ -63,11 +75,11 @@ QVariant Transfer::data(int role) const {
     case PriorityRole:
         return this->priority();
     case PriorityStringRole:
-        return Transfers::priorityString(this->priority());
+        return this->priorityString();
     case SizeRole:
         return this->size();
     case PositionRole:
-        return this->resumePosition() + this->downloadedBytes();
+        return this->position();
     case ProgressRole:
         return this->progress();
     case StatusRole:
@@ -84,6 +96,8 @@ QVariant Transfer::data(int role) const {
         return this->maximumConnections();
     case DownloadResumableRole:
         return this->downloadIsResumable();
+    case TransferCountRole:
+        return this->count();
     case IdRole:
         return this->id();
     case PackageIdRole:
@@ -94,6 +108,10 @@ QVariant Transfer::data(int role) const {
         return this->packageStatus();
     case RowNumberRole:
         return this->rowNumber();
+#ifdef QML_USER_INTERFACE
+    case ExpandedRole:
+        return this->expanded();
+#endif
     default:
         return QVariant();
     }
@@ -187,6 +205,11 @@ bool Transfer::setData(int role, const QVariant &value) {
         }
 
         break;
+#ifdef QML_USER_INTERFACE
+    case ExpandedRole:
+        this->setExpanded(value.toBool());
+        break;
+#endif
     default:
         return false;
     }
@@ -231,6 +254,27 @@ bool Transfer::isPackage() const {
     return (this->isValid()) && ((!this->parentTransfer()) || (!this->parentTransfer()->isValid()));
 }
 
+int Transfer::rowNumber() const {
+    return m_row;
+}
+
+void Transfer::setRowNumber(int row) {
+    m_row = row;
+}
+
+#ifdef QML_USER_INTERFACE
+bool Transfer::expanded() const {
+    return m_expanded;
+}
+
+void Transfer::setExpanded(bool expanded) {
+    if (expanded != this->expanded()) {
+        m_expanded = expanded;
+        emit expandedChanged();
+    }
+}
+#endif
+
 Transfer* Transfer::parentTransfer() const {
     return m_parent;
 }
@@ -280,7 +324,9 @@ Transfers::Status Transfer::packageStatus() const {
 void Transfer::setPackageStatus(Transfers::Status status) {
     if (status != this->packageStatus()) {
         m_packageStatus = status;
+#ifndef QML_USER_INTERFACE
         emit dataChanged(PackageStatusRole);
+#endif
     }
 }
 
@@ -332,7 +378,9 @@ void Transfer::setFileName(const QString &fileName) {
     if (fileName != this->fileName()) {
         m_fileName = fileName;
         m_fileName.replace(ILLEGAL_CHARS_RE, "_");
+#ifndef QML_USER_INTERFACE
         emit dataChanged(NameRole);
+#endif
     }
 }
 
@@ -355,7 +403,9 @@ QString Transfer::serviceName() const {
 void Transfer::setServiceName(const QString &name) {
     if (name != this->serviceName()) {
         m_serviceName = name;
+#ifndef QML_USER_INTERFACE
         emit dataChanged(NameRole);
+#endif
 
         if (name.isEmpty()) {
             this->setIconFileName(ICON_PATH + "qdl.png");
@@ -366,14 +416,14 @@ void Transfer::setServiceName(const QString &name) {
             ServicePlugin *plugin = PluginManager::instance()->getServicePlugin(name);
             this->setIconFileName(ICON_PATH + (!plugin ? "qdl.png" : plugin->iconName()));
 
-            if (plugin->maximumConnections() <= 0) {
+            if ((plugin) && (plugin->maximumConnections() <= 0)) {
                 // Plugin allows unlimited connections
-                this->setPreferredConnections(Settings::instance()->maximumConnectionsPerTransfer(), false);
                 this->setMaximumConnections(MAX_CONNECTIONS);
+                this->setPreferredConnections(Settings::instance()->maximumConnectionsPerTransfer(), false);
             }
             else {
-                this->setPreferredConnections(!plugin ? 1 : qMin(plugin->maximumConnections(), Settings::instance()->maximumConnectionsPerTransfer()), false);
                 this->setMaximumConnections(!plugin ? 1 : qMin(plugin->maximumConnections(), MAX_CONNECTIONS));
+                this->setPreferredConnections(!plugin ? 1 : qMin(plugin->maximumConnections(), Settings::instance()->maximumConnectionsPerTransfer()), false);
             }
         }
     }
@@ -386,7 +436,9 @@ QString Transfer::iconFileName() const {
 void Transfer::setIconFileName(const QString &fileName) {
     if (fileName != this->iconFileName()) {
         m_iconFileName = fileName;
+#ifndef QML_USER_INTERFACE
         emit dataChanged(IconRole);
+#endif
     }
 }
 
@@ -401,8 +453,11 @@ QString Transfer::category() const {
 void Transfer::setCategory(const QString &category) {
     if (category != this->category()) {
         m_category = category;
+#ifdef QML_USER_INTERFACE
+        emit categoryChanged();
+#else
         emit dataChanged(CategoryRole);
-
+#endif
         foreach (Transfer *transfer, m_transfers) {
             transfer->setCategory(category);
         }
@@ -416,14 +471,21 @@ Transfers::Priority Transfer::priority() const {
 void Transfer::setPriority(Transfers::Priority priority) {
     if (priority != this->priority()) {
         m_priority = priority;
+#ifdef QML_USER_INTERFACE
+        emit priorityChanged();
+#else
         emit dataChanged(PriorityRole);
-
+#endif
         foreach (Transfer *transfer, m_transfers) {
             transfer->setPriority(priority);
         }
     }
 }
 
+QString Transfer::priorityString() const {
+    return Transfers::priorityString(this->priority());
+}
+
 qint64 Transfer::size() const {
     return m_size;
 }
@@ -431,7 +493,11 @@ qint64 Transfer::size() const {
 void Transfer::setSize(qint64 size) {
     if ((size > 0) && (size != this->size())) {
         m_size = size;
+#ifdef QML_USER_INTERFACE
+        emit sizeChanged();
+#else
         emit dataChanged(SizeRole);
+#endif
     }
 }
 
@@ -442,20 +508,19 @@ qint64 Transfer::resumePosition() const {
 void Transfer::setResumePosition(qint64 position) {
     if (position != this->resumePosition()) {
         m_resumePosition = position;
+#ifdef QML_USER_INTERFACE
+        emit positionChanged();
+#else
         emit dataChanged(PositionRole);
-
+#endif
         if ((this->position() > 0) && (this->size() > 0)) {
             this->setProgress(position * 100 / this->size());
         }
     }
 }
 
-qint64 Transfer::downloadedBytes() const {
-    return m_downloadedBytes;
-}
-
 qint64 Transfer::position() const {
-    return this->resumePosition() + this->downloadedBytes();
+    return this->resumePosition() + m_downloadedBytes;
 }
 
 int Transfer::progress() const {
@@ -465,15 +530,19 @@ int Transfer::progress() const {
 void Transfer::setProgress(int progress) {
     if (progress != this->progress()) {
         m_progress = progress;
+#ifdef QML_USER_INTERFACE
+        emit progressChanged();
+#else
         emit dataChanged(ProgressRole);
+#endif
     }
 }
 
 int Transfer::speed() const {
     switch (this->status()) {
     case Transfers::Downloading:
-        if ((this->downloadedBytes() > 0) && (m_downloadTime.elapsed() > 0)) {
-            return this->downloadedBytes() / m_downloadTime.elapsed();
+        if ((m_downloadedBytes > 0) && (m_downloadTime.elapsed() > 0)) {
+            return m_downloadedBytes / m_downloadTime.elapsed();
         }
 
         break;
@@ -515,7 +584,11 @@ void Transfer::setStatus(Transfers::Status status) {
             break;
         }
 
+#ifdef QML_USER_INTERFACE
+        emit statusInfoChanged();
+#else
         emit dataChanged(StatusRole);
+#endif
         emit statusChanged(status);
     }
 }
@@ -532,13 +605,17 @@ QString Transfer::statusString() const {
 }
 
 QString Transfer::statusInfo() const {
-    return m_statuInfo;
+    return m_statusInfo;
 }
 
 void Transfer::setStatusInfo(const QString &info) {
     if (info != this->statusInfo()) {
-        m_statuInfo = info;
+        m_statusInfo = info;
+#ifdef QML_USER_INTERFACE
+        emit statusInfoChanged();
+#else
         emit dataChanged(StatusRole);
+#endif
     }
 }
 
@@ -559,6 +636,11 @@ void Transfer::setConvertToAudio(bool convert) {
     if (convert != this->convertToAudio()) {
         if ((!convert) || (this->convertibleToAudio())) {
             m_convert = convert;
+#ifdef QML_USER_INTERFACE
+            emit convertToAudioChanged();
+#else
+            emit dataChanged(ConvertToAudioRole);
+#endif
         }
     }
 }
@@ -570,7 +652,11 @@ int Transfer::preferredConnections() const {
 void Transfer::setPreferredConnections(int pref, bool overrideGlobalSetting) {
     if ((pref != this->preferredConnections()) && (pref > 0) && (pref < this->maximumConnections())) {
         m_preferredConnections = pref;
-
+#ifdef QML_USER_INTERFACE
+        emit preferredConnectionsChanged();
+#else
+        emit dataChanged(PreferredConnectionsRole);
+#endif
         if (overrideGlobalSetting) {
             this->disconnect(Settings::instance(), SIGNAL(maximumConnectionsPerTransferChanged(int,int)), this, SLOT(onMaximumConnectionsChanged(int,int)));
         }
@@ -620,7 +706,7 @@ QList<Connection*> Transfer::connections() const {
 }
 
 void Transfer::loadConnections() {
-    if (m_restoredConnections.isEmpty()) {
+    if (m_connections.isEmpty()) {
         this->addConnection(0, 0);
     }
     else {
@@ -633,16 +719,12 @@ void Transfer::loadConnections() {
         m_file.setMetaInfo(info);
         m_file.start(QThread::LowestPriority);
 
-        for (int i = 0; i < m_restoredConnections.size(); i++) {
-            this->addConnection(m_restoredConnections.at(i).first,
-                                m_restoredConnections.at(i).second,
-                                i < this->preferredConnections());
-        }
+        this->addConnections(this->preferredConnections());
     }
 }
 
 void Transfer::restoreConnection(qint64 start, qint64 end) {
-    m_restoredConnections.append(QPair<qint64, qint64>(start, end));
+    this->addConnection(start, end, false);
 }
 
 void Transfer::addConnection(qint64 start, qint64 end, bool startWhenAdded) {
@@ -682,6 +764,8 @@ void Transfer::addConnections(int count) {
 
     while ((added < count) && (i < m_connections.size())) {
         if (m_connections.at(i)->status() != Transfers::Downloading) {
+            m_connections.at(i)->setRequest(m_request);
+            m_connections.at(i)->setData(m_data);
             m_connections.at(i)->start();
             added++;
         }
@@ -729,16 +813,8 @@ void Transfer::removeConnections(int count) {
     }
 }
 
-int Transfer::rowNumber() const {
-    return m_row;
-}
-
-void Transfer::setRowNumber(int row) {
-    m_row = row;
-}
-
 void Transfer::createPackage() {
-    this->setPackageId(QString::number(QDateTime::currentMSecsSinceEpoch()));
+    this->setPackageId(QString::number(Utils::currentMSecsSinceEpoch()));
     this->setPackageSuffix(this->fileName().mid(this->fileName().lastIndexOf('.') + 1).toLower());
 
     QString packageName = this->fileName().left(this->fileName().lastIndexOf(QRegExp("p(ar|)t\\d+\\.", Qt::CaseInsensitive)));
@@ -774,6 +850,11 @@ void Transfer::addChildTransfer(Transfer *transfer) {
     m_transfers.append(transfer);
     transfer->setRowNumber(m_transfers.size() - 1);
     transfer->setParentTransfer(this);
+#ifdef QML_USER_INTERFACE
+    emit countChanged();
+#else
+    emit dataChanged(TransferCountRole);
+#endif
 }
 
 void Transfer::insertChildTransfer(int row, Transfer *transfer) {
@@ -784,6 +865,11 @@ void Transfer::insertChildTransfer(int row, Transfer *transfer) {
     for (int i = row + 1; i < m_transfers.size(); i++) {
         m_transfers.at(i)->setRowNumber(i);
     }
+#ifdef QML_USER_INTERFACE
+    emit countChanged();
+#else
+    emit dataChanged(TransferCountRole);
+#endif
 }
 
 Transfer* Transfer::removeChildTransfer(int row) {
@@ -793,7 +879,11 @@ Transfer* Transfer::removeChildTransfer(int row) {
         for (int i = row; i < m_transfers.size(); i++) {
             m_transfers.at(i)->setRowNumber(i);
         }
-
+#ifdef QML_USER_INTERFACE
+        emit countChanged();
+#else
+        emit dataChanged(TransferCountRole);
+#endif
         return transfer;
     }
 
@@ -820,6 +910,13 @@ void Transfer::start() {
         m_nam = NetworkAccessManager::create();
     }
 
+    qint64 bytes = m_downloadedBytes;
+
+    if (bytes > 0) {
+        m_downloadedBytes = 0;
+        this->setResumePosition(this->resumePosition() + bytes);
+    }
+
     if (this->serviceName().isEmpty()) {
         this->onDownloadRequestReady(QNetworkRequest(this->url()));
         return;
@@ -1050,15 +1147,21 @@ void Transfer::onCaptchaReady(const QByteArray &imageData) {
         QDeclarativeView *view = new QDeclarativeView;
         view->engine()->addImageProvider(QString("captcha"), new CaptchaImageProvider);
         view->rootContext()->setContextProperty("Utils", new QmlUtils(view));
-        view->setSource(QUrl::fromLocalFile("/opt/qdl/qml/harmattan/CaptchaDialog.qml"));
-        QGraphicsObject *dialog = view->rootObject();
-        dialog->setProperty("captchaImage", QString("image://captcha/" + imageData.toBase64()));
-        dialog->setProperty("timeOut", 120);
-        this->connect(dialog, SIGNAL(rejected()), view, SLOT(deleteLater()));
-        this->connect(dialog, SIGNAL(rejected()), this, SLOT(onCaptchaRejectedByUser()));
-        this->connect(dialog, SIGNAL(captchaResponseReady(QString)), view, SLOT(deleteLater()));
-        this->connect(dialog, SIGNAL(captchaResponseReady(QString)), this, SLOT(onCaptchaResponseReceived(QString)));
-        view->showFullScreen();
+        view->setSource(QUrl("qrc:/CaptchaDialog.qml"));
+
+        if (QGraphicsObject *dialog = view->rootObject()) {
+            dialog->setProperty("captchaImage", QString("image://captcha/" + imageData.toBase64()));
+            dialog->setProperty("timeOut", 120);
+            this->connect(dialog, SIGNAL(rejected()), view, SLOT(deleteLater()));
+            this->connect(dialog, SIGNAL(rejected()), this, SLOT(onCaptchaRejectedByUser()));
+            this->connect(dialog, SIGNAL(captchaResponseReady(QString)), view, SLOT(deleteLater()));
+            this->connect(dialog, SIGNAL(captchaResponseReady(QString)), this, SLOT(onCaptchaResponseReady(QString)));
+            view->showFullScreen();
+        }
+        else {
+            this->setStatusInfo(tr("Cannot load captcha dialog"));
+            this->setStatus(Transfers::Failed);
+        }
 #else
         QPixmap image;
 
@@ -1172,8 +1275,11 @@ void Transfer::onMetaInfoReady(MetaInfo info) {
 
 void Transfer::onBytesDownloaded(qint64 bytes) {
     m_downloadedBytes += bytes;
+#ifdef QML_USER_INTERFACE
+    emit positionChanged();
+#else
     emit dataChanged(PositionRole);
-
+#endif
     if (this->size() > 0) {
         this->setProgress(this->position() * 100 / this->size());
     }
@@ -1194,6 +1300,8 @@ void Transfer::onFileError() {
 
 void Transfer::onConnectionStatusChanged(Transfers::Status status) {
     switch (status) {
+    case Transfers::Downloading:
+        break;
     case Transfers::Completed:
         if (Connection* connection = qobject_cast<Connection*>(this->sender())) {
             this->onConnectionCompleted(connection);
@@ -1202,10 +1310,8 @@ void Transfer::onConnectionStatusChanged(Transfers::Status status) {
 
         break;
     case Transfers::Failed:
-        foreach (Connection *connection, m_connections) {
-            if (connection->status() != status) {
-                return;
-            }
+        if (this->activeConnections() > 0) {
+            return;
         }
 
         if (Connection* connection = qobject_cast<Connection*>(this->sender())) {
@@ -1213,7 +1319,6 @@ void Transfer::onConnectionStatusChanged(Transfers::Status status) {
         }
 
         break;
-
     case Transfers::Cancelled:
         foreach (Connection *connection, m_connections) {
             if (connection->status() != status) {
@@ -1287,7 +1392,15 @@ void Transfer::moveFiles() {
         }
     }
 
-    destinationDir.mkpath(destinationDir.path());
+    if (!destinationDir.mkpath(destinationDir.path())) {
+        destinationDir.setPath(Settings::instance()->downloadPath());
+
+        if (!destinationDir.mkpath(destinationDir.path())) {
+            this->setStatusInfo(tr("Cannot move downloaded files"));
+            this->setStatus(Transfers::Failed);
+            return;
+        }
+    }
 
     QDir downloadDir(this->downloadPath());
 
