@@ -23,6 +23,7 @@ class Transfer : public QObject
 
     Q_ENUMS(Roles)
 
+#ifdef QML_USER_INTERFACE
     Q_PROPERTY(QString id
                READ id
                CONSTANT)
@@ -46,34 +47,40 @@ class Transfer : public QObject
                CONSTANT)
     Q_PROPERTY(QString category
                READ category
-               WRITE setCategory)
+               WRITE setCategory
+               NOTIFY categoryChanged)
     Q_PROPERTY(Transfers::Priority priority
                READ priority
-               WRITE setPriority)
+               WRITE setPriority
+               NOTIFY priorityChanged)
+    Q_PROPERTY(QString priorityString
+               READ priorityString
+               NOTIFY priorityChanged)
     Q_PROPERTY(qint64 size
                READ size
-               NOTIFY dataChanged)
+               NOTIFY sizeChanged)
     Q_PROPERTY(qint64 position
                READ position
-               NOTIFY dataChanged)
+               NOTIFY positionChanged)
     Q_PROPERTY(int progress
                READ progress
-               NOTIFY dataChanged)
+               NOTIFY progressChanged)
     Q_PROPERTY(int speed
                READ speed
-               NOTIFY dataChanged)
+               NOTIFY speedChanged)
     Q_PROPERTY(Transfers::Status status
                READ status
                NOTIFY statusChanged)
     Q_PROPERTY(QString statusString
                READ statusString
-               NOTIFY statusChanged)
+               NOTIFY statusInfoChanged)
     Q_PROPERTY(QString statusInfo
                READ statusInfo
-               NOTIFY statusChanged)
+               NOTIFY statusInfoChanged)
     Q_PROPERTY(int preferredConnections
                READ preferredConnections
-               WRITE setPreferredConnections)
+               WRITE setPreferredConnections
+               NOTIFY preferredConnectionsChanged)
     Q_PROPERTY(int maximumConnections
                READ maximumConnections
                CONSTANT)
@@ -82,7 +89,8 @@ class Transfer : public QObject
                CONSTANT)
     Q_PROPERTY(bool convertToAudio
                READ convertToAudio
-               WRITE setConvertToAudio)
+               WRITE setConvertToAudio
+               NOTIFY convertToAudioChanged)
     Q_PROPERTY(bool downloadIsResumable
                READ downloadIsResumable
                CONSTANT)
@@ -95,6 +103,14 @@ class Transfer : public QObject
     Q_PROPERTY(QString packageSuffix
                READ packageSuffix
                CONSTANT)
+    Q_PROPERTY(int count
+               READ count
+               NOTIFY countChanged)
+    Q_PROPERTY(bool expanded
+               READ expanded
+               WRITE setExpanded
+               NOTIFY expandedChanged)
+#endif
 
 public:
     enum Roles {
@@ -114,11 +130,15 @@ public:
         PreferredConnectionsRole,
         MaximumConnectionsRole,
         DownloadResumableRole,
+        TransferCountRole,
         IdRole,
         PackageIdRole,
         PackageNameRole,
         PackageStatusRole,
         RowNumberRole
+#ifdef QML_USER_INTERFACE
+        ,ExpandedRole
+#endif
     };
 
     explicit Transfer(QObject *parent = 0);
@@ -137,6 +157,11 @@ public:
     int rowNumber() const;
     void setRowNumber(int row);
 
+#ifdef QML_USER_INTERFACE
+    bool expanded() const;
+    void setExpanded(bool expanded);
+#endif
+
     Transfer* parentTransfer() const;
     void setParentTransfer(Transfer *transfer);
 
@@ -178,14 +203,14 @@ public:
     Transfers::Priority priority() const;
     void setPriority(Transfers::Priority priority);
 
+    QString priorityString() const;
+
     qint64 size() const;
     void setSize(qint64 size);
 
     qint64 resumePosition() const;
     void setResumePosition(qint64 position);
 
-    qint64 downloadedBytes() const;
-
     qint64 position() const;
 
     int progress() const;
@@ -278,8 +303,22 @@ private slots:
     void moveFiles();
 
 signals:
-    void dataChanged(int role);
     void statusChanged(Transfers::Status status);
+#ifdef QML_USER_INTERFACE
+    void preferredConnectionsChanged();
+    void categoryChanged();
+    void priorityChanged();
+    void sizeChanged();
+    void progressChanged();
+    void positionChanged();
+    void speedChanged();
+    void convertToAudioChanged();
+    void statusInfoChanged();
+    void countChanged();
+    void expandedChanged();
+#else
+    void dataChanged(int role);
+#endif
 
 private:
     Transfer *m_parent;
@@ -311,14 +350,16 @@ private:
     qint64 m_speed;
     QTime m_downloadTime;
     Transfers::Status m_status;
-    QString m_statuInfo;
+    QString m_statusInfo;
     mutable bool m_convertible;
     mutable bool m_checkedIfConvertible;
     bool m_convert;
     int m_row;
     int m_preferredConnections;
     int m_maxConnections;
-    QList< QPair<qint64, qint64> > m_restoredConnections;
+#ifdef QML_USER_INTERFACE
+    bool m_expanded;
+#endif
     QList<Connection*> m_connections;
     QList<Transfer*> m_transfers;
     QStringList m_archivePasswords;
