@@ -4,8 +4,11 @@
 #include <QObject>
 #include <QQueue>
 #include <QUrl>
+#include <QStringList>
+#include <QMutex>
 
 class QNetworkReply;
+class UrlProcessor;
 
 class UrlRetriever : public QObject
 {
@@ -16,11 +19,14 @@ class UrlRetriever : public QObject
                NOTIFY progressChanged)
 
 public:
-    ~UrlRetriever();
-
     int progress() const;
-    QList<QUrl> results() const { return m_results; }
 
+    Q_INVOKABLE QList<QUrl> results() const;
+    Q_INVOKABLE QString resultsString() const;
+
+    static UrlRetriever* instance();
+
+public slots:
     void addUrlToQueue(const QUrl &url);
     void addUrlToQueue(const QString &url);
     void addUrlsToQueue(QList<QUrl> urls);
@@ -28,19 +34,18 @@ public:
     void parseUrlsFromText(const QString &text);
     void importUrlsFromTextFile(const QString &filePath);
 
-    static UrlRetriever* instance();
-
-public slots:
     void cancel();
     void clearResults();
 
 private:
     UrlRetriever();
+    ~UrlRetriever();
 
     void getWebPage(const QUrl &url);
 
 private slots:
     void checkWebPage();
+    void onUrlsProcessed(const QList<QUrl> &urls);
 
 signals:
     void busy(const QString &message, int numberOfOperations);
@@ -51,12 +56,32 @@ signals:
 private:
     static UrlRetriever *m_instance;
 
+    UrlProcessor *m_processor;
     QNetworkReply *m_reply;
+
     QQueue<QUrl> m_urlQueue;
     QList<QUrl> m_results;
+
     bool m_index;
     bool m_total;
     bool m_cancelled;
 };
 
+class UrlProcessor : public QObject
+{
+    Q_OBJECT
+
+    friend class UrlRetriever;
+
+private:
+    UrlProcessor();
+    ~UrlProcessor();
+
+private slots:
+    void processUrls(const QString &response);
+
+signals:
+    void urlsProcessed(const QList<QUrl> &urls);
+};
+
 #endif // URLRETRIEVER_H
